@@ -2,7 +2,7 @@
 * config_names.h
 
 
-Copyright (C) 2008-2010 Alessandro Vesely
+Copyright (C) 2008-2011 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -69,8 +69,8 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #define IPQBDB_DESCR_MAX_LENGTH 256
 #endif
 
-// The option aliases file, there is no other configuration except PCRE
-// (However, the DB_HOME environment variable may be set to a directory
+// The option aliases file, this is configuration, along with PCRE and exec
+// (In addition, the DB_HOME environment variable may be set to a directory
 // containing a DB_CONFIG file that will be read on automatic environment
 // creation. See Sleepycat docs, in particular for set_cachesize.)
 #if !defined IPQBDB_OPTION_FILE
@@ -82,6 +82,11 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #define IPQBDB_PCRE_FILE "/etc/ipqbdb-pcre.conf"
 #endif
 
+// The external tool command for killing existing connections
+#if !defined IPQBDB_CONNKILL_CMD
+#define IPQBDB_CONNKILL_CMD "/etc/ipqbdb-connkill.cmd"
+#endif
+
 // The socket where ibd-parse listens
 #if !defined IPQBDB_PARSE_SOCKET
 #define IPQBDB_PARSE_SOCKET "ibd-parse.sock"
