@@ -2,7 +2,7 @@
 * dbstruct.h
 
 
-Copyright (C) 2008-2010 Alessandro Vesely
+Copyright (C) 2008-2011 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -85,22 +85,8 @@ typedef struct ip_white_t
 	unsigned char chk;    // CHK_SIGNATURE, for loose type checking
 } ip_white_t;
 
-
-/*
-* This is the private data available via DB_ENV app_private
-*/
-typedef struct app_private
-{
-	int mode; // report_error
-	char const *err_prefix;
-} app_private;
-
-enum error_report_mode {error_report_stderr}; // or syslog facility
-
-#if defined __GNUC__
-__attribute__ ((format(printf, 3, 4)))
-#endif
-void report_error(app_private* ap, int priority, char const *fmt, ...);
+// used to be here
+#include "report_error.h"
 
 void fputs_database_help(void);
 char *database_fname(char const *name, app_private*);
@@ -123,13 +109,14 @@ int set_descr_cnt(DB *db1, int descr_id, int is_new_record,
 	ip_data_t const *deldata, time_t const *tloc);
 int open_descrdb(char *fname, DB_ENV *db_env, DB** db1, DB** db2);
 int check_whitelist(DB *, uint32_t ip, double *decay);
+
 typedef void (*write_block_cb)(uint32_t ip_addr,
 	ip_data_t *ip_data, ip_data_t *old_data, DB *db_d);
 enum
 { write_block_force_decay = 1, write_block_force_probability = 2,
 	write_block_force_reason = 4};
 int write_block(DB *db, uint32_t ip_addr, double decay, int reason_id,
-	int initial_probability, int force, write_block_cb verbose_fn, DB *db_d);
+	int* /* I/O */ probability, int force, write_block_cb verbose_fn, DB *db_d);
 
 void fputs_initial_count_help(void);
 
