@@ -3,7 +3,7 @@
 * written by ale in milano on 15 nov 2008
 * delete and/or list records in a block.db
 
-Copyright (C) 2008-2010 Alessandro Vesely
+Copyright (C) 2008-2011 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -56,6 +56,9 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include <arpa/inet.h>
 
 #include "ipv4_util.h"
+#include "initial_count.h"
+#include "percent_prob.h"
+#include "rehabilitated_prob.h"
 
 #include <pcre.h> // for grepping the reason table
 #include <assert.h>
@@ -692,24 +695,6 @@ static int get_probability(int *prob, char *probability)
 	return 0;
 }
 
-static double percent_prob(int p)
-{
-	return 100.0 * (double)p / ((double)RAND_MAX);
-}
-
-static int guess_initial_count(int prob)
-{
-	if (prob <= 0)
-		return 999;
-
-	unsigned count = 0;
-	unsigned long t = prob;
-	while ((t *= 2) <= RAND_MAX)
-		++count;
-
-	return count;
-}
-
 static int get_ip_address(ip4_range *ip)
 {
 	int err = parse_ip_address(ip_address, ip, NULL);
@@ -719,7 +704,6 @@ static int get_ip_address(ip4_range *ip)
 	return err;
 }
 
-
 static int
 do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 /*
@@ -735,6 +719,11 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 	unsigned total_selected = 0, total_del = 0, total_delfail = 0, total_recs = 0;
 	time_t const now = time(NULL), last_time = now - secs;
 
+	int max_count;
+	if (ban_opt)
+		max_count = max_initial_count();
+
+
 	char descr[IPQBDB_DESCR_MAX_LENGTH];
 
 	memset(&key, 0, sizeof key);
@@ -882,6 +871,17 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 					char buf1[TIME_STRING_BUFSIZE],
 						buf2[TIME_STRING_BUFSIZE],
 						buf3[TIME_STRING_BUFSIZE];
+
+					time_t delta = now - ip_data->last_update;
+					int reported_prob = ip_data->probability;
+					
+					if (reported_prob > 0 &&
+						UPDATE_TICK_TIME < delta &&
+						ip_data->decay > 0.0)
+							reported_prob = 
+								rehabilitated_prob(reported_prob,
+									delta, ip_data->decay);
+
 					if (verbose && list_opt && !total_selected)
 						printf("%15s %8s %7s %8s %10s %8s %10s %10s %10s %s\n",
 							"IP", "CREATED", "PROB.", "BLOCKED", "PACKETS",
@@ -889,10 +889,11 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 
 					get_descr(db_d, ip_data->reason_id, 0, descr, sizeof descr);
 					if (list_opt)
+					{
 						printf("%15s %8s %6.2f%% %8s %10d %8s %10g %10d %10d %s\n",
 							inet_ntoa(*(struct in_addr*)&cur_key),
 							time_string(ip_data->created, now, buf1),
-							percent_prob(ip_data->probability),
+							percent_prob(reported_prob),
 							time_string(ip_data->last_block, now, buf2),
 							ip_data->block_cnt,
 							time_string(ip_data->last_update, now, buf3),
@@ -900,12 +901,19 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 							ip_data->decay_add_cnt,
 							ip_data->caught_cnt,
 							descr);
+					}
 
+					/*
+					* creation and update times, as well as caught, blocked, and
+					* threshold counts are lost in this way.  However, ip, reasons,
+					* and decays are saved; probabilities will be nearly the same
+					* if results are sourced more or less at the same time.
+					*/
 					if (ban_opt)
 						printf("ibd-ban --ip-addr=%s --initial-count=%d"
 							" --initial-decay=%g --reason='%s'\n",
 							inet_ntoa(*(struct in_addr*)&cur_key),
-							guess_initial_count(ip_data->probability),
+							guess_initial_count(reported_prob, max_count),
 							ip_data->decay,
 							descr);
 				}
