@@ -2,7 +2,7 @@
 * ipq berkeley db daemon emitting verdicts - ibd-judge
 * written by ale in milano on 10 sep 2008
 
-Copyright (C) 2008-2010 Alessandro Vesely
+Copyright (C) 2008-2011 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -31,6 +31,7 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include <math.h>
 #include <errno.h>
 #include <ctype.h>
+#include <limits.h>
 
 #define __USE_BSD 1
 // for u_int in db.h
@@ -88,6 +89,9 @@ does not receive a stop of some sort.
 #include "config_names.h"
 #include "dbstruct.h"
 
+#include "percent_prob.h"
+#include "rehabilitated_prob.h"
+
 #include <assert.h>
 
 static app_private ap;
@@ -97,6 +101,9 @@ static char const err_prefix[] = "ibd-judge";
 // visible for TESTjudge
 int verbose = -1;
 
+// options for netfilter libraries
+static int netlink_bufsize, netlink_no_enobufs;
+
 // #if !defined NDEBUG
 #define TRACE(v, x...) \
 	while (ap.mode == 0 && verbose >= (v)) \
@@ -139,6 +146,7 @@ typedef struct queue_descr
 
 	// values set by parse_queue_descr
 	uint32_t mark_value;
+	unsigned int queue_maxlen;
 	uint16_t queue;
 	unsigned char dest;
 	unsigned char mark;
@@ -179,6 +187,14 @@ static int parse_queue_descr(char const *descr, queue_descr* out)
 					rtc = -1;
 				q.dest = 1;
 				break;
+			case 'l': //length
+				l = strtoul(descr, &t, 0);
+				if (l <= UINT_MAX)
+					q.queue_maxlen = (unsigned int)l;
+				else
+					rtc = -1;
+				descr = t;
+				break;
 			case 'm': // mark
 				l = strtoul(descr, &t, 0);
 				if (l <= UINT32_MAX)
@@ -271,11 +287,6 @@ packet_cb(struct nfq_q_handle *qh, struct nfgenmsg *nfmsg,
 	return 0;
 }
 
-static double percent_prob(int p)
-{
-	return 100.0 * (double)p / ((double)RAND_MAX);
-}
-
 static queue_descr default_queue[1], *queue_data = default_queue;
 static size_t queue_data_cnt = 1;
 
@@ -332,6 +343,25 @@ daemon_loop(struct nfq_handle *h, DB *db)
 		return 1;
 	}
 
+	if (netlink_bufsize)
+	{
+		unsigned int const read_size =
+			nfnl_rcvbufsiz(nfq_nfnlh(h), netlink_bufsize);
+		report_error(&ap, LOG_INFO, "netlink socket recv size size set to %u\n",
+			read_size);
+	}
+
+	if (netlink_no_enobufs)
+	{
+		int on = 1;
+		if (setsockopt(fd, SOL_NETLINK, NETLINK_NO_ENOBUFS, &on, sizeof on) != 0)
+		{
+			report_error(&ap, LOG_ERR, "cannot suppress ENOBUFS: %s\n",
+				strerror(errno));
+			// ignore the error
+		}
+	}
+
 	for (size_t i = 0; i < queue_data_cnt; ++i)
 	{
 		queue_descr *const qu = &queue_data[i];
@@ -352,17 +382,33 @@ daemon_loop(struct nfq_handle *h, DB *db)
 			nfq_destroy_queue(qh);
 			return 1;
 		}
-		
-		// TODO nfq_set_queue_maxlen(qh, configured_value), if needed
+
+		if (qu->queue_maxlen &&
+			nfq_set_queue_maxlen(qh, qu->queue_maxlen) < 0)
+		{
+			report_error(&ap, LOG_CRIT,
+				"can't set queue_maxlen to %u for queue #%d\n",
+				qu->queue_maxlen, qu->queue);
+			nfq_destroy_queue(qh);
+			return 1;
+		}
 
 		qu->cnt = &cb_cnt;
 		qu->qh = qh;
 		qu->deadbeef = 0xdeadbeef;
 	}
 
-	while (caught_signal == 0 &&
-		(rv = recv(fd, buf, sizeof(buf), 0)) >= 0)
+	while (caught_signal == 0)
 	{
+		if ((rv = recv(fd, buf, sizeof(buf), 0)) < 0)
+		{
+			if (errno != ENOBUFS)
+				break;
+			
+			report_error(&ap, LOG_ERR, "packet(s) lost\n");
+			continue;
+		}
+
 		TRACE(4, "PACKET RECV size=%d\n", rv);
 		nfq_handle_packet(h, buf, rv); // deliver to queue handlers
 
@@ -400,14 +446,15 @@ daemon_loop(struct nfq_handle *h, DB *db)
 						ip_data.last_block + RECENT_DROP_TIME >= now?
 							"again": "forever");
 				}
-				else // rehabilitate function
+				else // rehabilitate
 				{
+					// obvious: assert(ip_data.probability > 0);
 					time_t delta = now - ip_data.last_update;
 					if (UPDATE_TICK_TIME < delta && ip_data.decay > 0.0)
 					{
-						double d = (double)ip_data.probability *
-								exp2(-(double)delta / ip_data.decay);
-						unsigned int prob = isnormal(d)? (int)floor(0.5 + d): 0;
+						unsigned int const prob =
+							rehabilitated_prob(ip_data.probability,
+								delta, ip_data.decay);
 						if (prob < (unsigned) ip_data.probability)
 						{
 							ip_data.probability = prob;
@@ -432,8 +479,7 @@ daemon_loop(struct nfq_handle *h, DB *db)
 				}
 
 				/*
-				* re-insert the record
-				* in case it has been deleted since we read it
+				* update the record (re-insert it if it was deleted meanwhile)
 				*/
 				if (need_write &&
 					(rc = db->put(db, NULL, &key, &data, 0)) != 0)
@@ -569,6 +615,10 @@ static struct poptOption opttab[] =
 	"Be verbose", "level"},
 	{"no-db-cleanup", '\0', POPT_ARG_NONE, &no_cleanup_opt, 0,
 	"On exit don't cleanup environment (__db.00? files) if not still busy", NULL},
+	{"netlink-bufsize", '\0', POPT_ARG_INT, &netlink_bufsize, 0,
+	"Set nfnetlink buffer size", "bytes"},
+	{"netlink-no-enobufs", '\0', POPT_ARG_NONE, &netlink_no_enobufs, 0,
+	"Suppress error notifications for dropped packets", NULL},
 	{"version", 'V', POPT_ARG_NONE, &version_opt, 0,
 	"Print version number and exit", NULL},
 	{"help", 'h', POPT_ARG_NONE, &help_opt, 0,
@@ -632,13 +682,20 @@ int main(int argc, char const *argv[])
 			poptPrintHelp(opt, stdout, 0);
 			fputs_database_help();
 			fputs("\n"
-"Queue arguments can be specified as Qn[Mn][S|D], with no blanks between fields.\n"
+"Queue arguments can be specified as Qn[Mn][S|D][Ln], with no blanks, where\n"
+"QMSDL are (case insensitive) specifiers, and n are suitable decimal integers.\n"
 "Only one of S, for source, or D, for destination address, must be specified;\n"
-"S is assumed by default. If M, for mark, is specified, and the selected address\n"
-"is found to deserve being blocked, the packet is marked and accepted; otherwise\n"
-"it is dropped. For example, \"Q2M16D\" will check the destination address of\n"
-"packets on queue 2 and mark guilty packets with the value 16.\n\n"
-"If no queue argument is specified, \"Q0S\" is assumed.\n", stdout);
+"S is assumed by default. When the selected address is found to deserve being\n"
+"blocked, if M, for mark, is specified, the packet is marked and accepted, else\n"
+"it is dropped. L specifies the configured max length of the queue, in packets.\n"
+"\n"
+"For example, \"Q2M16DL10240\" will set queue_maxlen to 10240 for queue #2, and\n"
+"then check the destination addresses of those packets, marking guilty packets\n"
+"with the value 16, which can be tested with -m mark --mark 16 on a subsequent\n"
+"iptables table.\n"
+"\n"
+"Zero or more queue argumens can be specified. \"Q0S\" is assumed by default.\n",
+				stdout);
 			errs = 2;
 		}
 
