@@ -3,7 +3,7 @@
 * written by ale in milano on 28 sep 2008
 * read from a socket and extract pcre patterns to add to block.db
 
-Copyright (C) 2008-2010 Alessandro Vesely
+Copyright (C) 2008-2011 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -53,6 +53,8 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <pcre.h>
 
+#include <sys/wait.h>
+
 // for inet_aton
 #define _GNU_SOURCE 1
 #include <sys/socket.h>
@@ -63,6 +65,10 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include "config_names.h"
 #include "dbstruct.h"
 
+#include "initial_count.h"
+#include "percent_prob.h"
+#include "ext_exec.h"
+
 #include <assert.h>
 
 static char *trim(char *s, char *end)
@@ -123,6 +129,38 @@ int caught_signal;
 #define TRACE(x...)
 #endif
 
+static int live_children = 0, all_children = 0;
+static int sig_verbose = 0;
+
+static void child_reaper(int sig)
+{
+	int status;
+	pid_t child;
+
+	(void)sig;
+	
+	while ((child = waitpid(-1, &status, WNOHANG)) > 0)
+	{
+		--live_children;
+#if !defined(NDEBUG)
+		if (sig_verbose > 0 && ap.mode == error_report_stderr)
+		{
+			char buf[80];
+			unsigned s = snprintf(buf, sizeof buf,
+				"%s: %d exited, %d remaining\n",
+				ap.err_prefix,
+				(int)child, live_children);
+			if (s >= sizeof buf)
+			{
+				buf[sizeof buf - 1] = '\n';
+				s = sizeof buf;
+			}
+			write(2, buf, s);
+		}
+#endif
+	}
+}
+
 static void sig_catcher(int sig)
 {
 #if !defined(NDEBUG)
@@ -151,6 +189,11 @@ static int setsigs(void)
 	struct sigaction act;
 	memset(&act, 0, sizeof act);
 	sigemptyset(&act.sa_mask);
+		
+	act.sa_flags = SA_NOCLDSTOP;
+	act.sa_handler = child_reaper;
+	sigaction(SIGCHLD, &act, NULL);		
+
 	act.sa_flags = 0;
 	act.sa_handler = sig_catcher;
 
@@ -184,6 +227,7 @@ static char *pcre_name = IPQBDB_PCRE_FILE;
 static int version_opt, help_opt, no_cleanup_opt, chartable_opt, no_daemon_opt;
 static int exit_on_eof_opt, no_run_opt;
 static int verbose = -1;
+static int exec_connkill_opt;
 static struct poptOption opttab[] =
 {
 	{"db-block", 'b', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_block_name, 0,
@@ -200,12 +244,14 @@ static struct poptOption opttab[] =
 	"The default time for the block probability to halve (seconds)", "float"},
 	{"initial-count", 'c', POPT_ARG_INT|POPT_ARGFLAG_SHOW_DEFAULT, &initial_count, 0,
 	"Default initial full probability count", "integer"},
+	{"exec-connkill", 'e', POPT_ARG_NONE, &exec_connkill_opt, 0,
+	"Execute connection kill if probability reaches 100%", NULL},
 	{"exit-on-eof", '\0', POPT_ARG_NONE, &exit_on_eof_opt, 0,
 	"Exit when EOF is found on the fifo socket", NULL},
 	{"no-daemon", 'D', POPT_ARG_NONE, &no_daemon_opt, 0,
 	"Stay foreground and use stderr", NULL},
 	{"no-run", 'n', POPT_ARG_NONE, &no_run_opt, 0,
-	"Only check options and pcre action file", NULL},
+	"Only check options and config files", NULL},
 	{"verbose", 'v', POPT_ARG_INT|POPT_ARGFLAG_OPTIONAL, &verbose, 0,
 	"Be verbose", "level"},
 	{"pcre-use-locale", 'l', POPT_ARG_NONE, &chartable_opt, 0,
@@ -256,6 +302,7 @@ typedef struct initialization_stuff
 typedef struct runtime_stuff
 {
 	char *home_dir;
+	connkill_cmd *cmd;
 
 	DB_ENV *db_env;
 	DB *db1, *db2;
@@ -308,6 +355,7 @@ static void clear_run(runtime_stuff *run)
 	close_dbenv(run->db_env, !no_cleanup_opt);
 	free(run->db);
 	free(run->home_dir);
+	free(run->cmd);
 	clear_re_list(run->re_list);
 	memset(run, 0, sizeof *run);
 }
@@ -616,10 +664,9 @@ static int insert_pcre(initialization_stuff *ini,
 	if (count)
 	{
 		l = strtoul(count, &t, 0);
-		if (!(*t == 0 && l < CHAR_BIT * sizeof(int) &&
-			(r.init_prob = RAND_MAX >> (int)l) > 0))
+		r.init_prob = (l < UINT_MAX && *t == 0)? count2prob(l): 0;
+		if (r.init_prob == 0)
 		{
-			r.init_prob = 1;
 			report_error(&ap, *t == 0? LOG_NOTICE: LOG_ERR,
 				"initial count %s is %s: %s at line %d in %s\n",
 				count, *t == 0? "too hight": "bad",
@@ -628,6 +675,7 @@ static int insert_pcre(initialization_stuff *ini,
 			if (*t)
 				errs |= 2;
 		}
+		r.init_prob += 1; // rounded
 	}
 	else
 		r.init_prob = ini->default_ini_prob;
@@ -773,9 +821,11 @@ static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 	memset(&ini, 0, sizeof ini);
 
 	// set initial probability from prog args
-	if (!((unsigned)initial_count < CHAR_BIT * sizeof(int) &&
-		(ini.default_ini_prob = RAND_MAX >> initial_count) > 0))
-			ini.default_ini_prob = 1;
+	if ((ini.default_ini_prob = count2prob(initial_count)) == 0)
+		report_error( &ap, LOG_WARNING,
+			"bad initial count %d, default initial probability set to 1\n",
+			initial_count);
+	ini.default_ini_prob += 1; // rounded
 
 	if (fp == NULL)
 	{
@@ -852,6 +902,8 @@ static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 		return -1;
 	}
 
+	if (exec_connkill_opt)
+		run->cmd = read_connkill_cmd(&ap);
 	run->home_dir = ini.home_dir;
 	run->db_count = ini.fnames_count;
 	run->db_env = ini.db_env;
@@ -963,11 +1015,6 @@ static int get_ip_address(uint32_t* ip, char *dest, size_t dsize,
 	return 0;
 }
 
-static double percent_prob(int p)
-{
-	return 100.0 * (double)p / ((double)RAND_MAX);
-}
-
 static void display_written_block(uint32_t ip_addr,
 	ip_data_t *ip_data, ip_data_t *old_data, DB *is_null)
 {
@@ -985,6 +1032,27 @@ static void display_written_block(uint32_t ip_addr,
 		percent_prob(ip_data->probability));
 }
 
+static void spawn_exec(char *ip, connkill_cmd *cmd)
+{
+	pid_t pid = fork();
+	if (pid < 0)
+	{
+		report_error(&ap, LOG_CRIT,
+			"cannot fork: %s\n", strerror(errno));
+		return;
+	}
+
+	if (pid > 0) // parent
+	{
+		++live_children;
+		++all_children;
+		return;
+	}
+
+	run_exec(cmd, ip, &ap);
+	_exit(1); // error logged already
+}
+
 static int block_matched(runtime_stuff *run, re_list_t const * const r,
 	char const *str, size_t const len, int const rc)
 {
@@ -1059,12 +1127,15 @@ static int block_matched(runtime_stuff *run, re_list_t const * const r,
 
 		if (!dont_block)
 		{
+			int probability = r->init_prob;
 			rtc = write_block(run->db[r->db_ndx], ip_addr, decay,
-				r->reason, r->init_prob, 0,
+				r->reason, &probability, 0,
 				verbose? display_written_block: NULL, run->db1);
 
-			if (rtc < 0)
+			if (rtc != 0)
 				err = 1;
+			else if (probability >= RAND_MAX && run->cmd)
+				spawn_exec(dest32, run->cmd);
 		}
 	}
 
@@ -1277,7 +1348,7 @@ static int read_socket(socket_stuff *sock)
 			
 			char *fname = sock->name;
 			if (no_daemon_opt && exit_on_eof_opt)
-				fname = socket_name;
+				fname = socket_name; // predictable (base)name
 			report_error(&ap, LOG_NOTICE, "found EOF on %s\n", fname);
 			return 0;
 		}
@@ -1353,6 +1424,7 @@ int main(int argc, char const *argv[])
 		// popt sets verbose to 0 if no arg is given
 		// otherwise it stays -1
 		verbose += 1;
+		sig_verbose = verbose > 2;
 
 		ap.mode = error_report_stderr; // 0
 		ap.err_prefix = err_prefix;
@@ -1441,8 +1513,8 @@ int main(int argc, char const *argv[])
 							// through
 						default:
 							report_error(&ap, exit_rtc > 1? LOG_ALERT: LOG_NOTICE,
-								"exiting after signal %s\n",
-								strsignal(caught_signal));
+								"exiting after signal %s, %d/%d running child(ren)\n",
+								strsignal(caught_signal), live_children, all_children);
 							rtc = exit_rtc;
 							break;
 					}
