@@ -30,7 +30,6 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include <db.h>
 #include <limits.h>
 #include <syslog.h>
-#include <stdarg.h>
 #include <errno.h>
 #include <unistd.h>
 #include <sys/stat.h>
@@ -46,22 +45,6 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <assert.h>
 
-void report_error(app_private *ap, int pri, char const *fmt, ...)
-{
-	int mode = ap? ap->mode: error_report_stderr;
-	va_list al;
-	va_start(al, fmt);
-	if (mode == error_report_stderr)
-	{
-		if (ap && ap->err_prefix)
-			fprintf(stderr, "%s: ", ap->err_prefix);
-		vfprintf(stderr, fmt, al);
-	}
-	else // prefix already set in openlog
-		vsyslog(mode | pri, fmt, al);
-	va_end(al);
-}
-
 static int isdir(char const *p)
 {
 	struct stat st;
