@@ -1,7 +1,7 @@
 /*
 * opendb2.c
 
-Copyright (C) 2008-2010 Alessandro Vesely
+Copyright (C) 2008-2011 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -531,7 +531,7 @@ check_whitelist(DB *db, uint32_t ip_addr, double *decay)
 }
 
 int write_block(DB *db, uint32_t ip_addr, double decay, int reason_id,
-	int initial_probability, int force, write_block_cb verbose_fn, DB *db_d)
+	int *probability, int force, write_block_cb verbose_fn, DB *db_d)
 {
 	DBC *curs;
 	int rtc, rtc2 = 0, old = 0;
@@ -579,6 +579,8 @@ int write_block(DB *db, uint32_t ip_addr, double decay, int reason_id,
 					ip_data.probability *= 2;
 			else
 				ip_data.probability = RAND_MAX;
+			
+			const int initial_probability = *probability;
 			if (ip_data.probability < initial_probability ||
 				(force & write_block_force_probability) != 0)
 			{
@@ -598,6 +600,8 @@ int write_block(DB *db, uint32_t ip_addr, double decay, int reason_id,
 			ip_data.caught_cnt += 1;
 
 			rtc2 = curs->c_put(curs, &key, &data, DB_CURRENT);
+			if (rtc2 == 0)
+				*probability = ip_data.probability;
 		}
 	}
 	else if (rtc == DB_NOTFOUND)
@@ -605,7 +609,7 @@ int write_block(DB *db, uint32_t ip_addr, double decay, int reason_id,
 		rtc = 0;
 		ip_data.decay = decay;
 		ip_data.last_update = ip_data.created = now;
-		ip_data.probability = initial_probability;
+		ip_data.probability = *probability;
 		ip_data.reason_id = reason_id;
 		ip_data.caught_cnt = 1;
 		ip_data.chk = IPQBDB_CHK_SIGNATURE;
