@@ -1,7 +1,7 @@
 /* FAKED libnfqnetlink.c: generic library for FAKE access to nf_queue
  *
  * (C) 2005 by Harald Welte <laforge@gnumonks.org>
- * Copyright (C) 2010 Alessandro Vesely
+ * Copyright (C) 2010-2011 Alessandro Vesely
  *
 This file is part of Ipqbdb.
 
@@ -62,6 +62,7 @@ call to recv terminate with EBADF. nfq_fd will return -123 after that.
 #include <stdarg.h>
 #include <time.h>
 #include <errno.h>
+#include <limits.h>
 #include <wait.h>
 #include <sys/types.h>
 #include <netinet/in.h>
@@ -651,6 +652,7 @@ static void send_noop_packet(struct nfq_handle *h)
 
 struct nfnl_handle *nfq_nfnlh(struct nfq_handle *h)
 {
+	(void)h;
 	return NULL; // not used
 }
 
@@ -726,6 +728,7 @@ struct nfq_handle *nfq_open_nfnl(struct nfnl_handle *nfnlh)
 	}
 
 	return NULL;
+	(void)nfnlh;
 }
 
 int nfq_close(struct nfq_handle *h)
@@ -815,7 +818,7 @@ int nfq_handle_packet(struct nfq_handle *h, char *buf, int len)
 	
 	h->sv_sent = 0;
 	
-	while (len >= sizeof cmd && rtc >= 0)
+	while (len > 0 && (unsigned)len >= sizeof cmd && rtc >= 0)
 	{
 		int pkts = 0;
 		
@@ -946,6 +949,15 @@ int nfq_set_mode(struct nfq_q_handle *qh, u_int8_t mode, u_int32_t range)
 	return 0;
 }
 
+int nfq_set_queue_maxlen(struct nfq_q_handle *qh, u_int32_t queue_maxlen)
+{
+	if (get_queue_num(qh) < 0)
+		return -1;
+
+	print_out(qh->h, "nfq_set_queue_maxlen: %u\n", queue_maxlen);
+	return 0;
+}
+
 int nfq_set_verdict_mark(struct nfq_q_handle *qh, u_int32_t id,
 		u_int32_t verdict, u_int32_t mark,
 		u_int32_t datalen, unsigned char *buf)
@@ -956,6 +968,8 @@ int nfq_set_verdict_mark(struct nfq_q_handle *qh, u_int32_t id,
 		"nfq_set_verdict_mark: %d, %d, %d\n",
 		id, verdict, mark);
 	return 0;
+	(void)datalen;
+	(void)buf;
 }
 
 int nfq_set_verdict(struct nfq_q_handle *qh, u_int32_t id,
@@ -968,6 +982,8 @@ int nfq_set_verdict(struct nfq_q_handle *qh, u_int32_t id,
 		"nfq_set_verdict: %d, %d\n",
 		id, verdict);
 	return 0;
+	(void)data_len;
+	(void)buf;
 }
 
 /*************************************************************
@@ -1033,3 +1049,10 @@ int nfq_get_payload(struct nfq_data *pkt, char **data)
 	*data = pkt->packet.payload;
 	return sizeof pkt->packet.payload;
 }
+
+unsigned int nfnl_rcvbufsiz(const struct nfnl_handle *h, unsigned int size)
+{
+	(void)h;
+	return size*2;
+}
+
